/* 
 *  holly.c
 *  A -CD Player Hiding- v1.4.1 #14
 *  Copyright(C) 1998-2002 C-yan
 *  Last Update at 2002/10/09
 */

#include <string.h>
#include <windows.h>
#include "holly.h"

#define IDENT "HOLLY"

HMENU  hMenu;
HANDLE hMutex;
HICON  hIcon;

/* function's prototype declaration */
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
int ShowPlayer(int);
int IsNewPlayer(void);
void StartPlayer(void);
void SetTrayIcon(HWND, NOTIFYICONDATA *);
void Quit(NOTIFYICONDATA *);

int WINAPI WinMain(HINSTANCE hIns, HINSTANCE hPrevIns, LPSTR lpszArgv, int nDefaultWindowMode)
{
	HWND hWnd;
	WNDCLASSEX wcl;
	MSG Msg;
	
	if((hMutex = OpenMutex(MUTEX_ALL_ACCESS, FALSE, IDENT)) != NULL){ 
		CloseHandle(hMutex);
		return 1;
	}
	hMutex = CreateMutex(FALSE, 0, IDENT);
	hIcon = LoadIcon(hIns, MAKEINTRESOURCE(IDI_MAIN));
	hMenu = GetSubMenu(LoadMenu(hIns, MAKEINTRESOURCE(IDM_MAIN)), 0);
	
	wcl.hInstance = hIns;
	wcl.lpszClassName = IDENT;
	wcl.lpfnWndProc = WndProc;
	wcl.style = 0;
	wcl.hIcon = wcl.hIconSm = hIcon;
	wcl.hCursor = LoadCursor(NULL, IDI_APPLICATION);
	wcl.lpszMenuName = MAKEINTRESOURCE(IDM_MAIN);
	wcl.cbClsExtra = wcl.cbWndExtra = 0;
	wcl.hbrBackground = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
	wcl.cbSize = sizeof(WNDCLASSEX);
	
	if(RegisterClassEx(&wcl) == 0){
		return 1;
	}
	
	hWnd = CreateWindowEx(
		WS_EX_TOOLWINDOW,
		IDENT,
		IDENT,
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		HWND_DESKTOP,
		NULL,
		hIns,
		NULL
	);

	ShowWindow(hWnd, SW_HIDE);
	UpdateWindow(hWnd);

	while(GetMessage(&Msg, NULL, 0, 0)){
		TranslateMessage(&Msg);
		DispatchMessage(&Msg);
	}

	return Msg.wParam;
}

#define OLD_PLAYER_CLASS_NAME "SJE_CdPlayerClass"
#define NEW_PLAYER_CLASS_NAME "MMFRAME_MAIN"

int ShowPlayer(int nCmdShow)
{
	HANDLE hCDPlayer;
	char *PlayerClassName;
	
	if (IsNewPlayer() == 1){
		PlayerClassName = NEW_PLAYER_CLASS_NAME;
	} else {
		PlayerClassName = OLD_PLAYER_CLASS_NAME;
	}
	
	if((hCDPlayer = FindWindow(PlayerClassName, NULL)) != 0){
		ShowWindow(hCDPlayer, nCmdShow);
		return 1;
	} else {
		return 0;
	}
}

int IsNewPlayer(void)
{
  OSVERSIONINFO VersionInformation;
  
  VersionInformation.dwOSVersionInfoSize = sizeof(VersionInformation);
  GetVersionEx(&VersionInformation);

  if (VersionInformation.dwMajorVersion >= 5){   
  	return 1;  /* Windows2000 or later */ 
  } else {
  	return 0;  /* Window95/98/98SE (Me?) */
  }
}

#define PlayerExeName "\\cdplayer.exe"

void StartPlayer(void)
{
	STARTUPINFO SI;
	PROCESS_INFORMATION PI;
	char PlayerPath[MAX_PATH + 1];
	
	GetStartupInfo(&SI);
	if (IsNewPlayer() == 1){
		GetSystemDirectory(PlayerPath, MAX_PATH);
	} else {
		GetWindowsDirectory(PlayerPath, MAX_PATH);
	}
	strcat(PlayerPath, PlayerExeName);
	CreateProcess(PlayerPath, "", NULL, NULL, FALSE, 0, NULL, NULL, &SI, &PI);
}

#define TIP_STRING "A -CD Player Hiding-"

void SetTrayIcon(HWND hWnd, NOTIFYICONDATA *nIcon)
{
	nIcon->cbSize = sizeof(NOTIFYICONDATA);
	nIcon->hWnd = hWnd;
	nIcon->uID = 1;
	nIcon->uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	nIcon->uCallbackMessage = WM_TASKTRAY;
	nIcon->hIcon = (HICON)CopyImage(hIcon, IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), LR_COPYFROMRESOURCE);
	strcpy(nIcon->szTip, TIP_STRING);
	Shell_NotifyIcon(NIM_ADD, nIcon);
	DestroyIcon(nIcon->hIcon); 
}

void Quit(NOTIFYICONDATA *nIcon)
{
	ShowPlayer(SW_SHOW);
	Shell_NotifyIcon(NIM_DELETE, nIcon);
	ReleaseMutex(hMutex);
	PostQuitMessage(0);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	POINT point;
	static NOTIFYICONDATA nIcon;
	
	switch (uMsg){
	case WM_TASKTRAY:
		if(lParam == WM_RBUTTONDOWN){
			GetCursorPos(&point);
			SetForegroundWindow(hWnd);  //not need?
			TrackPopupMenu(hMenu, TPM_BOTTOMALIGN | TPM_RIGHTBUTTON, point.x, point.y, 0, hWnd, NULL);
		}
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam)){
		case IDM_VIEW:
			if (ShowPlayer(SW_SHOW) == 0){
				StartPlayer();
			}
			break;
		case IDM_HIDE:
			ShowPlayer(SW_HIDE);
			break;
		case IDM_QUIT:
			SendMessage(hWnd, WM_DESTROY, 0, 0);
			break;
		}
		break;
	case WM_CREATE:
		SetTrayIcon(hWnd, &nIcon);
		break;
	case WM_DESTROY:
		Quit(&nIcon);
		break;
	default:
		return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}
	return 0;
}